import ipaddress
import json
import http.client
import logging
import socket

import findssh
from findssh.coro import get_hosts as coro_get_hosts
from findssh.threadpool import get_hosts as threadpool_get_hosts
import log


class HTTPDetector:

    PORT = 80
    TIMEOUT = 1
    FINDSSH_SERVICE = "http"
    PATH = "/"
    RESPONSE_FIELD_NAME = "status"

    def __init__(self, parent=None, profile=None):
        self.logger = logging.getLogger('app.' + __name__)
        self.ip = findssh.getLANip()
        self.parent = parent
        self.profile = profile
        self.discovered_printers = []
    
    def filter_non_rrfw_hosts(self, hosts):
        filtered = []
        for host in hosts:
            host_ip = str(host[0])
            if self.parent_stop():
                break
            if self.check_ip(host_ip):
                filtered.append(host_ip)
        return filtered 

    def check_ip(self, host): 
        self.logger.debug("Detecting host:" + host)
        connection = http.client.HTTPConnection(host, self.PORT, timeout=self.TIMEOUT)
        self.logger.debug("Connecting")
        try:
            connection.connect()
            connection.request('GET', self.PATH)
            resp = connection.getresponse()
        except (socket.error, http.client.error) as e:
            self.logger.warning("Error on http detection request: " + str(e))
        else:
            self.logger.debug("Checking status")
            if resp.status == 200:
                self.logger.debug("Reading body")
                try:
                    answer_body = resp.read()
                except (socket.error, http.client.error) as e:
                    self.logger.warning("Error on http detection body read: " + str(e))
                else:
                    self.logger.debug("Decoding json")
                    try:
                        decoded = json.loads(answer_body)
                    except (ValueError, TypeError) as e:
                        self.logger.info("RRFW detector got non json response: " + str(e))
                    else:
                        self.logger.debug("Checking status dictionary")
                        if isinstance(decoded, dict):
                            test_field_contents = decoded.get(self.RESPONSE_FIELD_NAME)
                        # elif isinstance(decoded, list):
                        #     if decoded:
                        #         test_field_contents = decoded[0]
                        # elif isinstance(decoded, str):
                        #     test_field_contents = decoded
                        else:
                            test_field_contents = None
                        self.logger.debug('Got test field:' + str(test_field_contents))
                        return test_field_contents

    def parent_stop(self):
        return self.parent and self.parent.stop_flag

    def detect(self):
        network = findssh.netfromaddress(self.ip)
        if isinstance(network, ipaddress.IPv6Network):
            hosts = threadpool_get_hosts(network, self.PORT, self.FINDSSH_SERVICE, self.TIMEOUT)
        else:
            hosts = findssh.runner(coro_get_hosts, network, self.PORT, self.FINDSSH_SERVICE, self.TIMEOUT)
        self.logger.debug("All responded hosts:%s" % hosts)
        if self.profile:
            VID, PID = self.profile['vids_pids'][0]
        else:
            VID, PID = "XXXX", "XXXX" #dummy values only for debug mode
        discovered_printers = []
        for host in self.filter_non_rrfw_hosts(hosts):
            self.logger.debug("HOST: %s" % host)
            printer = {}
            printer['SNR'] = host
            printer['IP'] = host
            printer['VID'] = VID
            printer['PID'] = PID
            if printer not in discovered_printers:
                discovered_printers.append(printer)
        self.discovered_printers = discovered_printers
